#		       Welcome to the Space Template!
#
#     Contols are very simple:
#	Use Left and Right arrows to pan your ship
#	Use Up and Down arrows to control your ship's Tilt
#	Use the SPACE BAR to fire engines
#
///////////////////////////////////////////////////////////////////////////////////
// 	A4 main wdl
///////////////////////////////////////////////////////////////////////////////////
// The PATH keyword gives directories where game files can be found,
// relative to the level directory
PATH	"models";	// Path to model subdirectory - if any
PATH	"sounds";	// Path to sound subdirectory - if any
PATH	"bmaps";	// Path to graphics subdirectory - if any
PATH	"images";	// Path to images subdirectory
PATH	"..\\template";	// Path to WDL templates subdirectory


////////////////////////////////////////////////////////////////////////////
// The INCLUDE keyword can be used to include further WDL files,
// like those in the TEMPLATE subdirectory, with prefabricated actions
////////////////////////////////////////////////////////////////////////////
INCLUDE <movement.wdl>;
INCLUDE <messages.wdl>;
INCLUDE <particle.wdl>;
INCLUDE <menu.wdl>;

function key_init() { return; }	// dummy function, so we don't have to include doors.wdl

// Starting engine values
IFDEF LORES;
VAR VIDEO_MODE = 4;		// 320x240
IFELSE;
VAR VIDEO_MODE = 6;		// 640x480
ENDIF;

VAR VIDEO_DEPTH = 16;		// D3D, 16 bit resolution
VAR FPS_MAX = 40;		// 40 fps max

/////////////////////////////////////////////////////////////////
// define a splash screen with the required A4/A5 logo
bmap splashmap = <logodark.bmp>; // the default A4 logo in templates
panel splashscreen { bmap = splashmap; flags = REFRESH; }

// The MAIN function is called at game start and gets everything going

FUNCTION  MAIN()
{
// center the splash screen for non-640x480 resolutions
	splashscreen.pos_x = (screen_size.x - bmap_width(splashmap))/2;
	splashscreen.pos_y = (screen_size.y - bmap_height(splashmap))/2;
// set it visible
	splashscreen.visible = on;
// wait 2 frames until it is rendered and flipped to the foreground
	wait(2);

// now load the level
	LOAD_LEVEL <BIGSPACE.WMB>;	// The name of your level
// wait at least one frame until the level is loaded
	wait(1);
// wait the rest of the required second, then switch the splashscreen off.
//	waitt(8);
  	splashscreen.visible = off;

	LOAD_STATUS();			// restore global skills
}


//////////////////////////////////////////////////////////////////////////////
// 	SPACE SCRIPT BEGINS HERE
//////////////////////////////////////////////////////////////////////////////

// Sounds:
	SOUND MAIN_ENGINES,<ENGINE.WAV>;
	SOUND THRUSTERS,<THRUSTER.WAV>;
	VAR THRUST_HANDLE = 0;
	VAR ENGINE_HANDLE = 0;

// Synonyms:
	SYNONYM PLAYER_SHIP {TYPE ENTITY;}
	SYNONYM STAR_SPHERE {TYPE ENTITY;}



// The ACCEL_VECTOR tells the ship in which direction the thrust from the engines
// is coming. Because the ship was built with it's front pointing down the "X" axis, the
// thrust from the engine will always act on the "X" component of the ACCEL_VECTOR,
// which I have renamed ENGINE_THRUST for clarity.

	DEFINE ACCEL_VECTOR,SKILL1;
	DEFINE ENGINE_THRUST,SKILL1;


// While the ACCEL_VECTOR applies thrust relative to the SHIP'S orientation,
// the DRIFT_VECTOR applies the existing speed relative to the worlds orientation.

	DEFINE DRIFT_VECTOR,SKILL4;
	DEFINE DRIFT_VECTOR_X,SKILL4;
	DEFINE DRIFT_VECTOR_Y,SKILL5;
	DEFINE DRIFT_VECTOR_Z,SKILL6;

	DEFINE SHIP_ANGLES,SKILL7;	// Pan, Tilt and Roll speeds for ship
	DEFINE PAN_SPEED,SKILL7;	// How fast you are panning
	DEFINE TILT_SPEED,SKILL8;	// How fast you are tilting
	DEFINE ROLL_SPEED,SKILL9;	// How fast you are rolling
	DEFINE SPIN_RATE,SKILL10;	// The acceleration of your spin


// *** Note: While the coefficient of friction in space is for all practical purposes **ZERO**,
//	  using a small value in DECEL_RATE or putting a cap on the ships speed and spin can make
//	  it much easier to handle the ship or tune your game play. Think of it as the Navigation
//	  computer automatically taking the appropriate actions when the ship is not under active
//	  control.These fields are used when their corresponding flag is turned on.

	DEFINE DECEL_RATE,SKILL11;	   // How fast "Friction" (or retro-rockets!) will slow you down
	DEFINE MAX_SPIN_SPEED,SKILL12;     // Cap for  turn speed if LIMIT_TURN_SPEED flag is set
	DEFINE MAX_SHIP_SPEED,SKILL13;     // Cap for ship speed if LIMIT_TOP_SPEED flag is set

// interim counter for engine particle effect

define particle_rate,skill14;


//*** Define Flags for space ship flight characteristics
	DEFINE AUTO_SPIN_STOP,FLAG1;	//    These flags should all be off for
	DEFINE AUTO_DECEL,FLAG2;	//    maximum realism, but can be
	DEFINE LIMIT_TURN_SPEED,FLAG3;	//    useful to make a ship "Handle"
	DEFINE LIMIT_TOP_SPEED,FLAG4;	//    better.


DEFINE STAR_SKY,<STARS.MDL>;		// SPHERE MODEL WITH STARFIELD


// ************ Working Variables start here *******************
VAR SHIP_ANGS[3];
VAR ENGINE_WORK_SPEED[3];


// Camera Variables
  VAR CAMERA_TYPE;
  VAR CAMERA_SPOT[3];
  VAR WORKANG[3];




ACTION SPACE_SPHERE
{
	STAR_SPHERE = ME;

	WHILE (1)	// Constantly keep the player at the center of the universe
	{
		VEC_SET(MY.POS,PLAYER_SHIP.POS);
		WAIT 1;
	}
}

// hastily added particle effect for eye candy
function particle_thrustrun()
{
// decrease the size of the thrust particle
 	my_size -= 0.05*my_size*time;
// Remove old particles
	if(my_size < 10) { my_action = null; }
}

var my_blend[3];	// for older engines

function particle_thrust()
{
// place at random positions behind the engine
	mY_pos.x -= random(1)*player_ship.drift_vector_x*time;
	mY_pos.y -= random(1)*player_ship.drift_vector_y*time;
	mY_pos.z -= random(1)*player_ship.drift_vector_z*time;

// generate a backthrust
	my_speed.x = -engine_work_speed.x + random(0.5)-0.25;	// -0.25 to +0.25
	my_speed.y = -engine_work_speed.y + random(0.5)-0.25;
	my_speed.z = -engine_work_speed.x + random(0.5)-0.25;
	my_size = random(100)+50; 		// 100 to 150 in size

  	my_map = spark_map;
	my_flare = on;
	my_blend = on;

	my_color.red = 245;
	my_color.green = 100;
	my_color.blue = 50;

	my_action = particle_thrustrun;
}


ACTION PLAYER_SPACESHIP
{
// Set Camera mode
	CAMERA_TYPE = 1;     //**** 1 = Chase Cam, 2 = Cockpit, 3 = Stationary ****

// Set Synonym for future use and Zero out all ship speeds
 	PLAYER_SHIP = ME;
	MY.NARROW = ON;				// Use a smaller collision hull
	VEC_SET(MY.ACCEL_VECTOR,NULLVECTOR);	// No starting speed
	VEC_SET(MY.DRIFT_VECTOR,NULLVECTOR);	// No starting momentum
	CREATE STAR_SKY,MY.POS,SPACE_SPHERE;	// Initialize background stars


//*********************************************************
// Turn all special flight characteristics on to start. Set them to 0 to turn them off.
	MY.AUTO_SPIN_STOP = 1;		// Let the ship stop spinning by itself
	MY.AUTO_DECEL = 1;		// Let the ship slow down by using MY.DECEL_RATE
	MY.LIMIT_TURN_SPEED = 1;	// Limit how fast you spin based on MY.MAX_SPIN_SPEED
	MY.LIMIT_TOP_SPEED = 1;		// Ship "speedlimit" based on MY.MAX_SHIP_SPEED

// Starting flight Characteristics for ship:
	MY.ENGINE_THRUST = .5;		// Strength of the main engines
	MY.SPIN_RATE = .15;		// Strength of the steering thrusters
	MY.DECEL_RATE = .04;		// "Drift" deceleration rate
	MY.MAX_SPIN_SPEED = 5;		// Top speed for changes in ships orientation
	MY.MAX_SHIP_SPEED = 10;		// Top speed the ship can go
//*********************************************************


	WHILE (1) {

# *****Turning thruster logic here:
#
#	If you fire a thruster and you are using LIMIT_TURN_SPEEDS then check
#	    to make sure you are still under the speed limit.

		IF (KEY_CUL == 1)
		    {IF ((MY.LIMIT_TURN_SPEED == 0) ||
		           (MY.LIMIT_TURN_SPEED == 1) &&
		           (MY.PAN_SPEED < MY.MAX_SPIN_SPEED))
			{MY.PAN_SPEED += MY.SPIN_RATE * TIME;
			  IF (THRUST_HANDLE == 0)	// no sound playing
			      {PLAY_LOOP THRUSTERS,15;
			        THRUST_HANDLE = RESULT;}}}

		IF (KEY_CUR == 1)
		    {IF ((MY.LIMIT_TURN_SPEED == 0) ||
		           (MY.LIMIT_TURN_SPEED == 1) &&
		           (MY.PAN_SPEED > -MY.MAX_SPIN_SPEED))
			{MY.PAN_SPEED -= MY.SPIN_RATE * TIME;
			  IF (THRUST_HANDLE == 0)	// no sound playing
			      {PLAY_LOOP THRUSTERS,15;
			        THRUST_HANDLE = RESULT;}}}

		IF (KEY_CUU == 1)
		    {IF ((MY.LIMIT_TURN_SPEED == 0) ||
		           (MY.LIMIT_TURN_SPEED == 1) &&
		           (MY.TILT_SPEED < MY.MAX_SPIN_SPEED))
			{MY.TILT_SPEED += MY.SPIN_RATE * TIME;
			  IF (THRUST_HANDLE == 0)	// no sound playing
			      {PLAY_LOOP THRUSTERS,15;
			        THRUST_HANDLE = RESULT;}}}

		IF (KEY_CUD == 1)
		    {IF ((MY.LIMIT_TURN_SPEED == 0) ||
		           (MY.LIMIT_TURN_SPEED == 1) &&
		           (MY.TILT_SPEED > -MY.MAX_SPIN_SPEED))
			{MY.TILT_SPEED -= MY.SPIN_RATE * TIME;
			  IF (THRUST_HANDLE == 0)	// no sound playing
			      {PLAY_LOOP THRUSTERS,15;
			        THRUST_HANDLE = RESULT;}}}

		IF ((THRUST_HANDLE != 0) && // If sound is playing...
			(KEY_CUU == 0) &&
			(KEY_CUD == 0) &&   //      ...and...
			(KEY_CUL == 0) &&
			(KEY_CUR == 0))	    //  No arrow keys were pressed
				{STOP_SOUND THRUST_HANDLE ;// stop the sound
				 THRUST_HANDLE = 0;}

//******* If using Auto_Spin_Stop, Stop spinning when not actively stearing.

		IF (MY.AUTO_SPIN_STOP == 1)
			{STOP_ROTATION();}


//********Main Engine firing is here:

		IF (KEY_SPACE == 1) 		// if the space bar is pressed
		{
			IF (ENGINE_HANDLE == 0)	// If no sound playing, then play
			{
				PLAY_LOOP MAIN_ENGINES,25;
			   ENGINE_HANDLE = RESULT;
			}

	// 1) Set ENGINE_WORK_SPEED to the current thrust
	// 2) Vec_rotate will convert the thrust relative to the ship's orientation
//		   (which is only the x axis) and convert it into the
//		   X,Y,& Z components relative to the world's orientation.

			VEC_SET(ENGINE_WORK_SPEED,MY.ENGINE_THRUST);
			VEC_ROTATE(ENGINE_WORK_SPEED,MY.PAN);

	// If there is no speed limit, then apply the thrust, If there is a
//		speed limit, apply only the components of thrust that have not maxed out
			IF (MY.LIMIT_TOP_SPEED == 0)
				{VEC_ADD(MY.DRIFT_VECTOR,ENGINE_WORK_SPEED);}
			ELSE
				{IF (ABS(ENGINE_WORK_SPEED.X + MY.DRIFT_VECTOR_X) < MY.MAX_SHIP_SPEED)
				     {MY.DRIFT_VECTOR_X += ENGINE_WORK_SPEED.X;}

				 IF (ABS(ENGINE_WORK_SPEED.Y + MY.DRIFT_VECTOR_Y) < MY.MAX_SHIP_SPEED)
				     {MY.DRIFT_VECTOR_Y += ENGINE_WORK_SPEED.Y;}

				 IF (ABS(ENGINE_WORK_SPEED.Z + MY.DRIFT_VECTOR_Z) < MY.MAX_SHIP_SPEED)
	 			     {MY.DRIFT_VECTOR_Z += ENGINE_WORK_SPEED.Z;}
			}

// Ignite engines - send a speed corrected number of particles
			my.particle_rate += time * vec_length(my.drift_vector_x);
			while (my.particle_rate > 1)	// 1 particle per quant
			{
				ent_vertex(temp,354);	// first engine vertex
				emit(1,temp,particle_thrust);
				ent_vertex(temp,368);	// second engine vertex
				emit(1,temp,particle_thrust);
				my.particle_rate -= 1;
			}
		}
		ELSE
		{
			STOP_SOUND ENGINE_HANDLE ;// stop playing sound
			ENGINE_HANDLE = 0;
		}

	//**** Set it all in motion
		MY.ROLL_SPEED = 0;			// We don't use ROLL as an active control
		ROTATE ME,MY.SHIP_ANGLES,NULLVECTOR;	// Rotate the ship based on our pan and tilt rates

	// If you are not actively firing the engines, and Auto_decel is active, slow down the ship
		IF ((KEY_SPACE != 1) && (MY.AUTO_DECEL == 1))
			{CALL DECEL_SHIP;}

	// calculate a distance by multiplying the speed with the time
   	vec_set(temp,MY.DRIFT_VECTOR);
		vec_scale(temp,time);
 		MOVE ME,NULLVECTOR,temp; // Then move the ship according to distance

		POINT_CAMERA();				// Adjust the camera
 		wait(1);
	}
}


FUNCTION STOP_ROTATION()
{
//   **** If you are still rotating, apply a rotation in the opposite direction to slow down
//
	IF (PLAYER_SHIP.PAN_SPEED == 0)
	    {GOTO CHECKTILT;}
	ELSE
	    {IF ((KEY_CUL == 0) && (KEY_CUR == 0))
		{IF (PLAYER_SHIP.PAN_SPEED > 0)
		    {PLAYER_SHIP.PAN_SPEED -= PLAYER_SHIP.SPIN_RATE * TIME;}
		 ELSE
		    {PLAYER_SHIP.PAN_SPEED += PLAYER_SHIP.SPIN_RATE * TIME;}}
		 IF (ABS(PLAYER_SHIP.PAN_SPEED) < .02)
			{PLAYER_SHIP.PAN_SPEED = 0;}}

CHECKTILT:

	IF (PLAYER_SHIP.TILT_SPEED == 0)
	    {RETURN;}
	ELSE
	    {IF ((KEY_CUU == 0) && (KEY_CUD == 0))
		{IF (PLAYER_SHIP.TILT_SPEED > 0)
		    {PLAYER_SHIP.TILT_SPEED -= PLAYER_SHIP.SPIN_RATE * TIME;}
		 ELSE
		    {PLAYER_SHIP.TILT_SPEED += PLAYER_SHIP.SPIN_RATE * TIME;}}
		 IF (ABS(PLAYER_SHIP.TILT_SPEED) < .02)
			{PLAYER_SHIP.TILT_SPEED = 0;}}
}
}


ACTION DECEL_SHIP
{
# *** If you have momentum:
#    1) Set ENGINE_WORK_SPEED to have the decel rate as it's "X" component of thrust
#		(It's negative because we are counter-thrusting to slow down,
#		 not thrusting to speed up)
#    2) Figure out the pan and tilt angles that correspond to the existing drift
#		vector using Vec_to_angle.
#    3) Use Vec_Rotate to convert the thrust from the Ship's perspective to the real
#		world's perspective.
#    4) Use the Vec_Scale command to time correct these speeds based on frame rate
#    5) Add this new Deceleration Vector to the existing Drift Vector to slow down

	IF (VEC_LENGTH(PLAYER_SHIP.DRIFT_VECTOR) != 0)		// If I have momentum
		{VEC_SET(ENGINE_WORK_SPEED,NULLVECTOR);
		 ENGINE_WORK_SPEED.X = -PLAYER_SHIP.DECEL_RATE;
		 VEC_TO_ANGLE(SHIP_ANGS.PAN,PLAYER_SHIP.DRIFT_VECTOR);
		 VEC_ROTATE(ENGINE_WORK_SPEED,SHIP_ANGS.PAN);
		 VEC_SCALE(ENGINE_WORK_SPEED,TIME);
		 VEC_ADD(PLAYER_SHIP.DRIFT_VECTOR,ENGINE_WORK_SPEED);}
	ELSE
		{RETURN;}


# *** If DRIFT_VECTOR is close enough to zero, make it zero

	IF (VEC_LENGTH(PLAYER_SHIP.DRIFT_VECTOR) < .02)
	   	{VEC_SET(PLAYER_SHIP.DRIFT_VECTOR,NULLVECTOR);}
}


VAR C1[3];
VAR C2[3];
VAR TEMPA[3];
VAR CAMERA_WORK;


function POINT_CAMERA()
{
	IF (CAMERA_TYPE == 2)
		{GOTO COCKPIT;}

	IF (CAMERA_TYPE == 3)
		{GOTO STATIONARY;}

// Begin arbitrary angle camera code:
//		(NOTE: The Chase Cam portion of this script was donated by JCL.
//		 It is currently somewhat rough, and I don't understand enough
//		 about how it works to attempt to explain it. Please direct
//		 questions about this section of code to him.
	CAMERA.GENIUS = NULL;			// Camera won't ignore anyone
	VEC_SET(WORKANG.PAN,PLAYER_SHIP.PAN);
	CAMERA_SPOT.X = -100;			// Relative displacement of camera to
	CAMERA_SPOT.Y = -10;			//     ship when in chase mode
	CAMERA_SPOT.Z = 20;

	VEC_SET(CAMERA.X,CAMERA_SPOT.X);
	VEC_ROTATE(CAMERA.X,PLAYER_SHIP.PAN);
	VEC_ADD(CAMERA.X,PLAYER_SHIP.X);

// For normal spaceships you don't need an arbitrary angle camera -
// the camera looks straight ahead. So the following arbitrary angle
// camera code is IFDEF'd out.
IFDEF ARBITRARY_ANGLE;

// SET CAMERA PAN AND TILT ANGLES
	VEC_SET (TEMPA.X,PLAYER_SHIP.X);
	VEC_SUB (TEMPA.X,CAMERA.X);
	VEC_TO_ANGLE(CAMERA.PAN,TEMPA);


// SET CAMERA ROLL ANGLES
	C1.X = TEMPA.Y;
	C1.Y = -TEMPA.X;
	C1.Z = 0;

	C2.X = -CAMERA_SPOT.Y;
	C2.Y = CAMERA_SPOT.X;
	C2.Z = 0;

	VEC_ROTATE(C2,PLAYER_SHIP.PAN);

// GET THE ANGLE DIFFERENCE BETWEEN BOTH VECTORS USING DOT PRODUCT
	TEMPA = C1.X*C2.X +C1.Y*C2.Y;
	tempa /= VEC_LENGTH(C1);
	tempa /= VEC_LENGTH(C2);

	IF (C2.Z < 0)
		{CAMERA.ROLL = ACOS(TEMPA);}
	ELSE
		{CAMERA.ROLL = -ACOS(TEMPA);}

IFELSE;	// not ARBITRARY_ANGLE
// straight ahead looking chase camera code
// with a little angle delay
//	if (time < 1) {
//		camera.pan += ang(player_ship.pan - camera.pan)*0.9*time;
//		camera.tilt += ang(player_ship.tilt - camera.tilt)*0.9*time;
//		camera.roll += ang(player_ship.roll - camera.roll)*0.9*time;
//	} else {
	vec_set(camera.pan,player_ship.pan);
ENDIF;
	RETURN;

//Cockpit Camera starts here
COCKPIT:
	CAMERA.GENIUS = PLAYER_SHIP;		// Camera won't see parts the ship
	VEC_SET(CAMERA.X,PLAYER_SHIP.X);	// Camera is where the ship is
	VEC_SET(CAMERA.PAN,PLAYER_SHIP.PAN);	// Camera looks in the same direction as ship
	RETURN;

//Stationary Camera Starts here
STATIONARY:
	CAMERA.GENIUS = NULL;			// Camera won't ignore anyone
	VEC_SET(CAMERA_WORK,PLAYER_SHIP.X);	// Point Camera at player
	VEC_SUB(CAMERA_WORK,CAMERA.X);
	VEC_TO_ANGLE(CAMERA.PAN,CAMERA_WORK); // now camera looks at the player
	RETURN;
}


/////////////  SPACE SCRIPT ENDS HERE //////////////////////